/* Copyright (c) 2006 Rice University */
/* All Rights Reserved */
/* This code is covered by the Rice-WARP license */
/* See http://warp.rice.edu/license/ for details */


#include "xmk.h"
#include <string.h>
#include <stdlib.h>
#include <os_config.h>
#include <sys/process.h>
#include <pthread.h>
#include "xparameters.h"
#include "stdio.h"
#include "xutil.h"
#include "warplib.h"
#include "xgpio_l.h"


//========================================================================
// This system will launch 2 initial threads of different priorities.
// It will then create a thread from within the software.
// This version of the demo function on Priority Scheduling (SCHED_PRIO).
// This processor execution is done by order of highest priority.
// the lower the priority number, the higher the order of priority
//
// The system does NOT utilize advanced OS functionality
// like interrupts, semaphores, mutex locks, messaging, etc.
// It is meant to serve as a very basic intro to xilkernel.
//
//
// Written by:	Scott Novich (snovich@rice.edu)
// for Nokia and Rice
//
//========================================================================

// the following declarations are neccessary for thread creation
static pid_t mainproc_pid; // an identifier used for killing the lower priority process
static pid_t init_pid; // an identifier for the kill higher priority thread attempt
static pthread_t newthread1; // an identifier for creating a new pthread
static pthread_attr_t thread1attr; // an attribute identifer used for creating the new pthread
static struct sched_param newthreadsched; // used for giving our new thread a priority level



//====================================================
//
// this is one of the systems main initial threads loaded when xilkernel_main is called
// this pthread-type is automatically initialized by xilkernel by simply defining it
// in the static_pthread_table under config_pthread_support of OS and Libraries under
// software platform settings.
// The thread has been given a priority of 1 (highest). The pthread table is the list
// of threads/processes to begin when the kernel STARTS. It's not meant to be a lazy
// way of creating generic threads (which involves a few function calls).

void* somenewthread(void *arg)
{
	print("\r\n\r\n -- I'm a new thread that's NOT hardcoded into Xilkernel's Software Platform Settings.");
	print("-- I'm done for now, let's go into permanent hibernation :) --");
	while(1)
	{
	;
	}
}

void* INITPROC(void *notused)
{
	init_pid = get_currentPID();
	int i;
	print("\r\n -- this is the initial process. It has the highest priority. When it finishes, we'll launch the main shell.\r\n");
	print("\r\n...turning on the WARP LEDs to act as some generic initialization process...\r\n");
	for(i = 0; i < 10 ; i++)
	{
		XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, 0x1);
		usleep(150000);
		XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, 0x2);
		usleep(150000);
		XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, 0x4);
		usleep(150000);
		XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, 0x8);
		usleep(150000);
		XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, 0x4);
		usleep(150000);
		XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, 0x2);
		usleep(150000);
	}

	// the following snippet can be used to kill the other proc if scheduling
	// is Round Robin (RR-based) instead of Priority based. You can set scheduling
	// type in the scheduling section of the OS/Drivers in Software Platform Settings
	//
	//print("\r\nattempting to kill the 7-segment display process...\r\n");
	//if (kill(mainproc_pid) != 0) {
     //       xil_printf ("\r\n\r\n Error during kill operation...\r\n");
   //}
	print("\r\n...we're done, time for mainproc!!!...\r\n");
}

// the following thread is also a system initial thread. The only difference is that it's of lower
// priority.
void* MAINPROC(void *alsonotused)
{
	int j = 0;
	int genericvalue;
	mainproc_pid = get_currentPID();
	// some running process that will be stopped by the initproc
	for(j = 0; j < 10 ; j++)
	{

		XGpio_mSetDataReg(XPAR_LED_7SEGMENT_BASEADDR, 1,WarpLib_segmap('0'));
		XGpio_mSetDataReg(XPAR_LED_7SEGMENT_1_BASEADDR, 1,WarpLib_segmap('A'));
		usleep(100000);
		XGpio_mSetDataReg(XPAR_LED_7SEGMENT_BASEADDR, 1,WarpLib_segmap('1'));
		XGpio_mSetDataReg(XPAR_LED_7SEGMENT_1_BASEADDR, 1,WarpLib_segmap('B'));
		usleep(100000);
		XGpio_mSetDataReg(XPAR_LED_7SEGMENT_BASEADDR, 1,WarpLib_segmap('2'));
		XGpio_mSetDataReg(XPAR_LED_7SEGMENT_1_BASEADDR, 1,WarpLib_segmap('C'));
		usleep(100000);
		XGpio_mSetDataReg(XPAR_LED_7SEGMENT_BASEADDR, 1,WarpLib_segmap('3'));
		XGpio_mSetDataReg(XPAR_LED_7SEGMENT_1_BASEADDR, 1,WarpLib_segmap('D'));
		usleep(100000);
		XGpio_mSetDataReg(XPAR_LED_7SEGMENT_BASEADDR, 1,WarpLib_segmap('4'));
		XGpio_mSetDataReg(XPAR_LED_7SEGMENT_1_BASEADDR, 1,WarpLib_segmap('E'));
		usleep(100000);
		XGpio_mSetDataReg(XPAR_LED_7SEGMENT_BASEADDR, 1,WarpLib_segmap('5'));
		XGpio_mSetDataReg(XPAR_LED_7SEGMENT_1_BASEADDR, 1,WarpLib_segmap('F'));
	}

	print("\r\n...main process finished! If I were cool I'd write a shell here...\r\n");
	print("\r\n Anyways let's create and launch a new THREAD that WASN'T hardcoded into xilkernel under Software Platform Settings...\r\n\r\n");

	// the following 4 function calls are all that's technically needed to launch a thread
	// the declaration for their parameters is defined above as an example
	// also note:
	// newthread1, thread1attr, and newthreadsched can be RECYCLED to create more threads.\
	// these threads can also be placed in separate C files as long as they're included
	// in the sources for this application project. No associated header file is required if it is a thread.
	pthread_attr_init(&thread1attr);
	newthreadsched.sched_priority = 2;
	pthread_attr_setschedparam(&thread1attr,&newthreadsched);
	genericvalue = pthread_create(&newthread1, &thread1attr, (void*)somenewthread, NULL); // NULL is a placeholder. It's used if the thread function must take in an argument

}

int main () {
   print("-- Entering main() --\r\n");
	print("-- press any key to continue--\r\n\r\n");
	getchar();
	xilkernel_main();
	print("-- Exiting main() --\r\n");
   return 0;
}

